from icemac.ab.calendar.interfaces import IEvent
from icemac.addressbook.interfaces import IEntity
from zope.testbrowser.browser import LinkNotFoundError
import pytest
import zope.component.hooks


def test_calendar__CalendarView__1(address_book, FieldFactory, browser):
    """It allows to select fields for display in the calendar."""
    field = FieldFactory(
        address_book, IEvent, u'Int', u'reservations').__name__
    browser.login('cal-editor')
    browser.open(browser.CALENDAR_MASTERDATA_URL)
    browser.getLink('Calendar view').click()
    assert browser.CALENDAR_MASTERDATA_EDIT_DISPLAY_URL == browser.url
    browser.getControl('Additional event fields').displayValue = [
        'persons',
        'reservations',
    ]
    browser.getControl('Apply').click()
    assert 'Data successfully updated.' == browser.message
    browser.open(browser.CALENDAR_MASTERDATA_EDIT_DISPLAY_URL)
    assert browser.getControl('Additional event fields').displayValue == [
        'persons',
        'reservations',
    ]
    # It does not break on selected but deleted user defined field:
    with zope.component.hooks.site(address_book):
        event_entity = IEntity(IEvent)
        event_entity.removeField(event_entity.getRawField(field))
    browser.open(browser.CALENDAR_MASTERDATA_EDIT_DISPLAY_URL)
    assert browser.getControl('Additional event fields').displayValue == [
        'persons',
    ]


def test_calendar__CalendarView__2(address_book, browser):
    """It is not shown on calendar master data for a calendar visitor."""
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_MASTERDATA_URL)
    with pytest.raises(LinkNotFoundError):
        browser.getLink('Calendar view')


def test_calendar__CalendarView__3(address_book, browser):
    """It is not accessible for a calendar visitor."""
    browser.login('cal-visitor')
    browser.assert_forbidden(browser.CALENDAR_MASTERDATA_EDIT_DISPLAY_URL)



from icemac.addressbook.i18n import _
import grokcore.component as grok
import icemac.ab.calendar.interfaces
import icemac.addressbook.browser.breadcrumb


class CalendarBreadCrumb(icemac.addressbook.browser.breadcrumb.Breadcrumb):
    """Breadcrumb for the calendar."""

    grok.adapts(
        icemac.ab.calendar.interfaces.ICalendar,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)

    title = _('Calendar')



from icemac.addressbook.i18n import _
import grokcore.component as grok
import icemac.ab.calendar.interfaces
import icemac.addressbook.utils
import persistent
import zope.catalog.interfaces
import zope.container.btree
import zope.container.contained
import zope.interface
import zope.lifecycleevent
import zope.schema.fieldproperty


@zope.interface.implementer(icemac.ab.calendar.interfaces.ICategories)
class CategoryContainer(zope.container.btree.BTreeContainer):
    """A container for calendar event categories."""


@zope.interface.implementer(icemac.ab.calendar.interfaces.ICategory)
class Category(persistent.Persistent,
               zope.container.contained.Contained):
    """A category of an event."""

    zope.schema.fieldproperty.createFieldProperties(
        icemac.ab.calendar.interfaces.ICategory)

    def __repr__(self):
        """Human readable representation of the object."""
        return '<Category title={0!r}>'.format(
            self.title.encode('ascii', 'replace'))


unique_titles = icemac.addressbook.utils.unique_by_attr_factory(
    'title', _(u'This category already exists.'))


@grok.subscribe(icemac.ab.calendar.interfaces.ICategory,
                zope.lifecycleevent.IObjectModifiedEvent)
def changed(obj, event):
    """Update the catalog index if a category's title has changed."""
    # The notation of of this function is meant to have no code branches
    # because this cannot be easily tested and the other branches do not get
    # excecuted in real live.
    changed = [
        desc
        for desc in event.descriptions
        if (desc.interface == icemac.ab.calendar.interfaces.ICategory
            and 'title' in desc.attributes)]
    for x in changed:
        catalog = zope.component.getUtility(
            zope.catalog.interfaces.ICatalog)
        catalog.updateIndex(catalog.get('keywords'))



from icemac.addressbook.i18n import _
import collections
import gocept.reference.field
import icemac.ab.calendar.interfaces
import icemac.addressbook.interfaces
import icemac.addressbook.sources
import zc.sourcefactory.basic
import zope.component
import zope.interface


class IEventViewContainer(zope.interface.Interface):
    """Container for event view configurations."""


class StartDateSource(icemac.addressbook.sources.TitleMappingSource):
    """Source mapping a relative number of days to a text."""

    _mapping = collections.OrderedDict((
        (-28, _('4 weeks in past')),
        (-21, _('3 weeks in past')),
        (-14, _('2 weeks in past')),
        (-7, _('1 week in past')),
        (-3, _('3 days in past')),
        (-1, _('1 day in past')),
        (0, _('current day')),
        (1, _('1 day in future')),
        (3, _('3 days in future')),
        (7, _('1 week in future')),
        (14, _('2 weeks in future')),
        (21, _('3 weeks in future')),
        (28, _('4 weeks in future')),
    ))


start_date_source = StartDateSource()


class DurationSource(icemac.addressbook.sources.TitleMappingSource):
    """Source a number of days to a text."""

    _mapping = collections.OrderedDict((
        (7, _('1 week')),
        (14, _('2 weeks')),
        (21, _('3 weeks')),
        (28, _('4 weeks')),
    ))


duration_source = DurationSource()


class IEventViewConfiguration(zope.interface.Interface):
    """Configuration of an view of events."""

    title = zope.schema.TextLine(
        title=_('title'),
        description=_('Shown on the event views page in the drop down.'))

    start = zope.schema.Choice(
        title=_('start date'),
        description=_(
            'The start date used in the view will be differing from the'
            ' current date by the chosen period of time.'),
        source=start_date_source,
    )

    duration = zope.schema.Choice(
        title=_('duration'),
        description=_('This number of days is shown in the view.'),
        source=duration_source,
    )

    categories = gocept.reference.field.Set(
        title=_('categories'),
        description=_(
            'Show in the view only events having one of these categories.'),
        required=False,
        value_type=zope.schema.Choice(
            title=_('event category'),
            source=icemac.ab.calendar.interfaces.category_source))

    fields = zope.schema.List(
        title=_('show fields'),
        description=_('Additional event fields to be shown in the view.'),
        required=False,
        value_type=zope.schema.Choice(
            source=icemac.ab.calendar.interfaces.event_fields_source))


class EventViewConfigurationSource(zc.sourcefactory.basic.BasicSourceFactory):
    """Source containing all event view configurations."""

    def getValues(self):
        container = zope.component.getUtility(IEventViewContainer)
        return sorted(container.values(), key=lambda x: x.title)

    def getTitle(self, value):
        return icemac.addressbook.interfaces.ITitle(value)


class IEventViews(zope.interface.Interface):
    """Listing of event view configurations."""

    views = zope.schema.Choice(
        title=u'known views',
        source=EventViewConfigurationSource(),
    )


class IEventData(zope.interface.Interface):
    """Date of a single event which can be rendered in the event view."""

    datetime = zope.interface.Attribute('datetime.datetime object')

    whole_day = zope.interface.Attribute(
        'Event is the whole day, so do not display time.')

    title = zope.interface.Attribute(
        'Category resp. alternative title of the event.')

    fields = zope.interface.Attribute(
        'List of values of the fields selected in the event view config.')



from icemac.ab.calendar.browser.calendar import EventDescription
import mock
import pytest


@pytest.fixture('session')
def EventDescriptionFactory(DateTime, zcmlS):
    """Create an EventDescription object."""
    def get_event_description(event, **kw):
        """Get an icemac.ab.calendar.browser.calendar.EventDescription.

        time_tuple ... `now` if empty.
        **kw ... attributes to be set on the event(!).
        """
        event.datetime = DateTime.now
        for key, value in kw.items():
            setattr(event, key, value)
        ICalendarDisplaySettings = (
            'icemac.ab.calendar.interfaces.ICalendarDisplaySettings')
        get_time_zone_name = (
            'icemac.addressbook.preferences.utils.get_time_zone_name')
        with mock.patch(ICalendarDisplaySettings) as ICalendarDisplaySettings,\
                mock.patch('icemac.ab.calendar.interfaces.ICalendar'),\
                mock.patch(get_time_zone_name, return_value='UTC'):
            ICalendarDisplaySettings.event_additional_fields = ()
            return EventDescription(event)
    return get_event_description



from cStringIO import StringIO
import gocept.month
import grokcore.component as grok
import icemac.ab.calendar.browser.base
import icemac.ab.calendar.browser.renderer.interfaces
import icemac.addressbook.browser.interfaces
import zope.i18n


@grok.implementer(icemac.ab.calendar.browser.renderer.interfaces.IRenderer)
class Calendar(grok.MultiAdapter,
               icemac.ab.calendar.browser.base.View):
    """Base of calendar view."""

    grok.baseclass()
    grok.adapts(
        gocept.month.IMonth,
        icemac.addressbook.browser.interfaces.IAddressBookLayer,
        list)

    def __init__(self, month, request, events):
        self.request = request
        self.month = month
        self.events = events
        self._fd = StringIO()

    def write(self, string, *args, **kw):
        """Store a string which might contain % marks which get replaced."""
        text = string % args
        # We have to encode the text here as the used cStringIO does not
        # support unicode characters outside ASCII:
        self._fd.write(text.encode('utf-8'))
        if kw.pop('newline', True):
            self._fd.write('\n')

    def read(self):
        """Get the stored information back as unicode."""
        return self._fd.getvalue().decode('utf-8')

    def update(self):
        pass

    def render(self):
        raise NotImplementedError()

    def translate(self, message_id):
        return zope.i18n.translate(message_id, context=self.request)

    def __call__(self):
        self.update()
        return self.render()



from zope.testbrowser.browser import LinkNotFoundError
from zope.security.interfaces import Unauthorized
import pytest


EVENT_CATEGORY_ADD_TEXT = 'event category'


def test_category__Table__1(address_book, browser):
    """It allows to navigate to the category list view."""
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_MASTERDATA_URL)
    browser.getLink('Event categories').click()
    assert browser.CALENDAR_CATEGORIES_LIST_URL == browser.url


def test_category__Table__2(address_book, browser):
    """It renders a message if there are no categories yet."""
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_CATEGORIES_LIST_URL)
    assert 'No event categories defined yet.' in browser.contents


def test_category__Table__3(address_book, browser):
    """It renders no add link for a calendar visitor."""
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_CATEGORIES_LIST_URL)
    with pytest.raises(LinkNotFoundError):
        browser.getLink(EVENT_CATEGORY_ADD_TEXT)


def test_category__Table__4(address_book, browser):
    """It prevents access for anonymous."""
    browser.handleErrors = False  # needed to catch exception
    with pytest.raises(Unauthorized):
        browser.open(browser.CALENDAR_CATEGORIES_LIST_URL)


def test_category__Add__1(address_book, browser):
    """It allows to add a new category in the list."""
    browser.login('cal-editor')
    browser.open(browser.CALENDAR_CATEGORIES_LIST_URL)
    browser.getLink(EVENT_CATEGORY_ADD_TEXT).click()
    assert browser.CALENDAR_CATEGORY_ADD_URL == browser.url
    browser.getControl('event category').value = 'birthday'
    browser.getControl('Add').click()
    assert '"birthday" added.' == browser.message
    # The new category shows up in the list:
    assert '>birthday<' in browser.contents


def test_category__Add__2(address_book, CategoryFactory, browser):
    """It prevents adding a new category with an already existing title."""
    CategoryFactory(address_book, u'birthday')
    browser.login('cal-editor')
    browser.open(browser.CALENDAR_CATEGORY_ADD_URL)
    browser.getControl('event category').value = 'birthday'
    browser.getControl('Add').click()
    assert 'There were some errors.' in browser.contents
    assert 'This category already exists.' in browser.contents


def test_category__Add__3(address_book, browser):
    """It is not accessible for a calendar visitor."""
    browser.login('cal-visitor')
    browser.assert_forbidden(browser.CALENDAR_CATEGORY_ADD_URL)


def test_category__Edit__1(address_book, CategoryFactory, browser):
    """It allows to edit a category."""
    CategoryFactory(address_book, u'birthday')
    browser.login('cal-editor')
    browser.open(browser.CALENDAR_CATEGORIES_LIST_URL)
    browser.getLink('birthday').click()
    assert browser.CALENDAR_CATEGORY_EDIT_URL == browser.url
    assert 'birthday' == browser.getControl('event category').value
    browser.getControl('event category').value = 'wedding day'
    browser.getControl('Apply').click()
    assert 'Data successfully updated.' == browser.message
    # The changed category name shows up in the list:
    assert 'wedding day' in browser.contents


def test_category__Edit__2(address_book, CategoryFactory, browser):
    """It prevents changing a category title to an existing one."""
    CategoryFactory(address_book, u'birthday')
    CategoryFactory(address_book, u'wedding day')
    browser.login('cal-editor')
    browser.open(browser.CALENDAR_CATEGORY_EDIT_URL)
    browser.getControl('event category').value = 'wedding day'
    browser.getControl('Apply').click()
    assert 'There were some errors.' in browser.contents
    assert 'This category already exists.' in browser.contents


def test_category__Edit__3(address_book, CategoryFactory, browser):
    """It allows a calendar visitor only to see the category data.

    But he cannot change or delete them.
    """
    CategoryFactory(address_book, u'foo')
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_CATEGORY_EDIT_URL)
    # There are no fields and no delete button:
    assert (['form.buttons.apply', 'form.buttons.cancel'] ==
            browser.all_control_names)


def test_category__Delete__1(address_book, CategoryFactory, browser):
    """It allows to delete a category."""
    CategoryFactory(address_book, u'birthday')
    browser.login('cal-editor')
    browser.open(browser.CALENDAR_CATEGORY_EDIT_URL)
    browser.getControl('Delete').click()
    assert browser.CALENDAR_CATEGORY_DELETE_URL == browser.url
    assert ('Do you really want to delete this event category?' in
            browser.contents)
    browser.getControl('Yes').click()
    assert '"birthday" deleted.' == browser.message


def test_category__Delete__2(
        address_book, CategoryFactory, EventFactory, browser):
    """It assures that a used category cannot be deleted."""
    EventFactory(
        address_book, category=CategoryFactory(address_book, u'birthday'))
    browser.login('cal-editor')
    browser.open(browser.CALENDAR_CATEGORY_DELETE_URL)
    with pytest.raises(LookupError):
        browser.getControl('Delete')


def test_category__Delete__3(address_book, CategoryFactory, browser):
    """It is not accessible for a calendar visitor."""
    CategoryFactory(address_book, u'foo')
    browser.login('cal-visitor')
    browser.assert_forbidden(browser.CALENDAR_CATEGORY_DELETE_URL)



from ..install import update_calendar_infrastructure
import icemac.addressbook.generations.utils


@icemac.addressbook.generations.utils.evolve_addressbooks
def evolve(addressbook):
    """Add event views infrastructure."""
    update_calendar_infrastructure(addressbook)



from __future__ import unicode_literals
from .interfaces import EventViewConfigurationSource


def test_interface__EventViewConfigurationSource__getValue__1(
        address_book, AddressBookFactory, EventViewConfigurationFactory):
    """It returns all created EventViewConfiguration objects of the site.

    The values are returned alphabetically sorted by the title of the
    EventViewConfiguration objects.
    """
    evc2 = EventViewConfigurationFactory(address_book, '2 weeks')
    evc1 = EventViewConfigurationFactory(address_book, '1 week')
    evc3 = EventViewConfigurationFactory(address_book, '3 weeks')
    ab2 = AddressBookFactory('ab2')
    EventViewConfigurationFactory(ab2, '1 month')
    source = EventViewConfigurationSource().factory
    assert [evc1, evc2, evc3] == list(source.getValues())


def test_interface__EventViewConfigurationSource__getTitle__1(
        address_book, EventViewConfigurationFactory):
    """It returns the title of the EventViewConfiguration."""
    evc = EventViewConfigurationFactory(address_book, '2 weeks')
    source = EventViewConfigurationSource().factory
    assert '2 weeks' == source.getTitle(evc)



from icemac.ab.calendar.category import Category
import icemac.ab.calendar.category
import icemac.ab.calendar.interfaces
import icemac.ab.calendar.testing
import icemac.addressbook.conftest
import icemac.addressbook.testing
import icemac.addressbook.utils
import pytest
import zope.component.hooks
import zope.publisher.browser


# Fixtures to create objects:


@pytest.fixture(scope='session')
def CategoryFactory():
    """Create an event category in the calendar."""
    def create_category(address_book, title, **kw):
        parent = address_book.calendar_categories
        with zope.component.hooks.site(address_book):
            name = icemac.addressbook.utils.create_and_add(
                parent, Category, title=title, **kw)
        return parent[name]
    return create_category


@pytest.fixture(scope='session')
def EventFactory():
    """Create an event in the calendar."""
    def create_event(address_book, **kw):
        return icemac.addressbook.testing.create(
            address_book, address_book.calendar,
            icemac.ab.calendar.interfaces.IEvent, **kw)
    return create_event


@pytest.fixture(scope='session')
def RecurringEventFactory():
    """Create a recurring event in its container."""
    def create_recurring_event(address_book, **kw):
        return icemac.addressbook.testing.create(
            address_book, address_book.calendar_recurring_events,
            icemac.ab.calendar.interfaces.IRecurringEvent, **kw)
    return create_recurring_event


# generally usable helper fixtures:


@pytest.fixture(scope='session')
def DateTime():
    """Fixture to ease handling of datetime objects."""
    return icemac.ab.calendar.testing.DateTimeClass()


@pytest.fixture('function')
def utc_time_zone_pref(TimeZonePrefFactory):
    """Set the time zone in the preferences to UTC."""
    return TimeZonePrefFactory('UTC')



from __future__ import unicode_literals

from ..event import StartColumn
from icemac.ab.calendar.event import BaseEvent
from zope.testbrowser.browser import LinkNotFoundError
from zope.security.interfaces import Unauthorized
import pytest
import pytz


RECURRING_EVENT_ADD_TEXT = 'recurring event'


def test_event__Table__1(address_book, browser):
    """It allows to navigate to the recurring events list view."""
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_MASTERDATA_URL)
    browser.getLink('Recurring Events').click()
    assert browser.CALENDAR_RECURRING_EVENTS_LIST_URL == browser.url


def test_event__Table__2(address_book, browser):
    """It renders a message if there are no recurring events yet."""
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_RECURRING_EVENTS_LIST_URL)
    assert 'No recurring events defined yet.' in browser.contents


def test_event__Table__3(address_book, browser):
    """It renders no add link for a calendar visitor."""
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_RECURRING_EVENTS_LIST_URL)
    with pytest.raises(LinkNotFoundError):
        browser.getLink(RECURRING_EVENT_ADD_TEXT)


def test_event__Table__4(address_book, browser):
    """It prevents access for anonymous."""
    browser.handleErrors = False  # needed to catch exception
    with pytest.raises(Unauthorized):
        browser.open(browser.CALENDAR_RECURRING_EVENTS_LIST_URL)


def test_event__StartColumn__renderCell__1(
        DateTime, RequestFactory, utc_time_zone_pref):
    """It renders date and time on a non-whole-day event."""
    event = BaseEvent()
    event.datetime = DateTime(
        2016, 3, 10, 8, 32, tzinfo=pytz.timezone('Europe/Berlin'))
    event.whole_day_event = False
    col = StartColumn(None, RequestFactory(), None)
    assert '16/03/10 07:32' == col.renderCell(event)


def test_event__StartColumn__renderCell__2(DateTime, RequestFactory):
    """It renders only the date a whole-day event."""
    event = BaseEvent()
    event.datetime = DateTime(2016, 3, 10, 8, 32)
    event.whole_day_event = True
    col = StartColumn(None, RequestFactory(), None)
    assert '16/03/10' == col.renderCell(event)


def test_event__StartColumn__renderCell__3(DateTime, RequestFactory):
    """It returns en empty string if no `datetime` is set."""
    event = BaseEvent()
    col = StartColumn(None, RequestFactory(), None)
    assert '' == col.renderCell(event)


def test_event__Add__1(address_book, CategoryFactory, DateTime, browser):
    """It allows to add recurring events to the list."""
    CategoryFactory(address_book, 'birthday')
    browser.login('cal-editor')
    browser.open(browser.CALENDAR_RECURRING_EVENTS_LIST_URL)
    browser.getLink(RECURRING_EVENT_ADD_TEXT).click()
    assert browser.CALENDAR_RECURRING_EVENT_ADD_URL == browser.url
    browser.getControl('event category').getControl('birthday').selected = True
    browser.getControl('date').value = DateTime.format_date(DateTime.now)
    browser.getControl('time').value = '21:45'
    browser.getControl('recurrence end').value = '2055 1 1 '
    browser.getControl(name='form.buttons.add').click()
    assert '"birthday" added.' == browser.message
    assert browser.CALENDAR_RECURRING_EVENTS_LIST_URL == browser.url
    browser.reload()  # get rid of the flash message
    # The new recurring event shows up in the list:
    assert 'birthday' in browser.contents


def test_event__Add__2(address_book, browser):
    """It is not accessible for a calendar visitor."""
    browser.login('cal-visitor')
    browser.assert_forbidden(browser.CALENDAR_RECURRING_EVENT_ADD_URL)


def test_event__Edit__1(
        address_book, RecurringEventFactory, CategoryFactory, DateTime,
        browser):
    """It allows to edit a recurring event."""
    CategoryFactory(address_book, 'birthday')
    RecurringEventFactory(
        address_book, alternative_title='wedding day', datetime=DateTime.now,
        period='weekly')
    browser.login('cal-editor')
    browser.open(browser.CALENDAR_RECURRING_EVENTS_LIST_URL)
    browser.getLink('wedding day').click()
    assert browser.CALENDAR_RECURRING_EVENT_EDIT_URL == browser.url
    assert 'wedding day' == browser.getControl('alternative title').value
    browser.getControl('alternative title').value = ''
    browser.getControl('event category').getControl('birthday').selected = True
    browser.getControl('Apply').click()
    assert 'Data successfully updated.' == browser.message
    # The changed event name shows up in the list:
    assert 'birthday' in browser.contents


def test_category__Edit__2(
        address_book, RecurringEventFactory, DateTime, browser):
    """It allows a calendar visitor only to see the recurring event data.

    But he cannot change or delete them.
    """
    RecurringEventFactory(
        address_book, alternative_title='foo', datetime=DateTime.now,
        period='weekly')
    browser.login('cal-visitor')
    browser.open(browser.CALENDAR_RECURRING_EVENT_EDIT_URL)
    # There are no fields and no delete button:
    assert (['form.buttons.apply', 'form.buttons.cancel'] ==
            browser.all_control_names)


def test_event__Delete__1(
        address_book, RecurringEventFactory, DateTime, browser):
    """It recurring_event_can_be_deleted."""
    RecurringEventFactory(
        address_book, alternative_title='birthday', datetime=DateTime.now)
    browser.login('cal-editor')
    browser.open(browser.CALENDAR_RECURRING_EVENT_EDIT_URL)
    browser.getControl('Delete').click()
    assert browser.CALENDAR_RECURRING_EVENT_DELETE_URL == browser.url
    assert ('Do you really want to delete this recurring event?' in
            browser.contents)
    browser.getControl('Yes').click()
    assert '"birthday" deleted.' == browser.message


def test_category__Delete__2(
        address_book, RecurringEventFactory, DateTime, browser):
    """It is not accessible for a calendar visitor."""
    RecurringEventFactory(
        address_book, alternative_title='birthday', datetime=DateTime.now)
    browser.login('cal-visitor')
    browser.assert_forbidden(browser.CALENDAR_RECURRING_EVENT_DELETE_URL)



import pytest


@pytest.mark.webdriver
def test_startpage__calendar__1_webdriver(address_book, webdriver):
    """It redirects to the calendar if this is set on the address book."""
    ab = webdriver.address_book
    webdriver.login('mgr', ab.ADDRESS_BOOK_EDIT_URL)
    ab.startpage = 'Calendar'
    ab.submit('apply')
    webdriver.open(ab.ADDRESS_BOOK_DEFAULT_URL)
    assert webdriver.calendar.CALENDAR_MONTH_OVERVIEW_URL == webdriver.path



from .model import EventViewConfiguration
import icemac.addressbook.utils
import pytest
import zope.component.hooks


@pytest.fixture(scope='session')
def EventViewConfigurationFactory():
    """Create an event view configuration in its container."""
    def create_eventview_configuration(address_book, title, **kw):
        parent = address_book.calendar_eventviews
        with zope.component.hooks.site(address_book):
            name = icemac.addressbook.utils.create_and_add(
                parent, EventViewConfiguration, title=title, **kw)
        return parent[name]
    return create_eventview_configuration



import icemac.ab.calendar.category
import icemac.ab.calendar.interfaces
import icemac.ab.calendar.testing
import icemac.addressbook.conftest
import icemac.addressbook.testing
import icemac.addressbook.utils
import icemac.recurrence.conftest
import pytest


pytest_plugins = (
    'icemac.addressbook.fixtures',
    'icemac.ab.calendar.fixtures',
)


# Fixtures to set-up infrastructure which are usable in tests:


@pytest.yield_fixture(scope='function')
def address_book(addressBookConnectionF):
    """Get the address book with calendar as site."""
    for address_book in icemac.addressbook.conftest.site(
            addressBookConnectionF):
        yield address_book


@pytest.fixture(scope='function')
def browser(browserWsgiAppS):
    """Fixture for testing with zope.testbrowser."""
    assert icemac.addressbook.testing.CURRENT_CONNECTION is not None, \
        "The `browser` fixture needs a database fixture like `address_book`."
    return icemac.ab.calendar.testing.Browser(wsgi_app=browserWsgiAppS)


# generally usable helper fixtures:


@pytest.fixture(scope='function')
def sitemenu(browser):
    """Helper fixture to test the selections in the site menu.

    USABLE IN TESTS.

    """
    return icemac.addressbook.testing.SiteMenu


@pytest.fixture(scope='function')
def assert_address_book(address_book):
    """Fixture returning an object providing a custom address book asserts."""
    return icemac.addressbook.testing.AddressBookAssertions(address_book)


# Infrastructure fixtures


@pytest.yield_fixture(scope='session')
def zcmlS():
    """Load calendar ZCML on session scope."""
    layer = icemac.addressbook.testing.SecondaryZCMLLayer(
        'Calendar', __name__, icemac.ab.calendar)
    layer.setUp()
    yield layer
    layer.tearDown()


@pytest.yield_fixture(scope='session')
def zodbS(zcmlS):
    """Create an empty test ZODB."""
    for zodb in icemac.addressbook.testing.pyTestEmptyZodbFixture():
        yield zodb


@pytest.yield_fixture(scope='session')
def addressBookS(zcmlS, zodbS):
    """Create an address book for the session."""
    for zodb in icemac.addressbook.conftest.pyTestAddressBookFixture(
            zodbS, 'CalendarS'):
        yield zodb


@pytest.yield_fixture(scope='function')
def addressBookConnectionF(addressBookS):
    """Get the connection to the right demo storage."""
    for connection in icemac.addressbook.conftest.pyTestStackDemoStorage(
            addressBookS, 'CalendarF'):
        yield connection



from ..browser.event import EVENT_CONFIRMATION_FIELDS, EventFields
from icemac.addressbook.i18n import _
import grokcore.component as grok
import icemac.ab.calendar.interfaces
import icemac.ab.calendar.masterdata.breadcrumb
import icemac.addressbook.browser.base
import icemac.addressbook.browser.breadcrumb
import icemac.addressbook.browser.metadata
import icemac.addressbook.browser.table
import icemac.addressbook.interfaces
import icemac.recurrence
import z3c.table.column


class RecurringEventsBreadCrumb(
        icemac.ab.calendar.masterdata.breadcrumb.CalendarMDChildBreadcrumb):
    """Breadcrumb for recurring events."""

    grok.adapts(
        icemac.ab.calendar.interfaces.IRecurringEvents,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)

    title = _('Recurring Events')


class RecurrenceColumn(z3c.table.column.I18nGetAttrColumn):
    """Column displaying the concrete recurrence of an event."""

    header = _('recurrence period')

    def getValue(self, item):
        recurring = icemac.recurrence.get_recurring(
            item.datetime, item.period)
        return recurring.info


class PersonsColumn(z3c.table.column.Column):
    """Column displaying all persons assigned to an event."""

    header = _('persons')

    def renderCell(self, item):
        return ', '.join(item.listPersons())


class StartColumn(z3c.table.column.GetAttrColumn):
    """Column which renders the start date.

    It localizes its datetime to selected timezone.
    It omits the time part of whole day events.
    """

    attrName = 'datetime'
    header = _('datetime')

    def getFormatter(self, obj):
        if obj.whole_day_event:
            category = 'date'
        else:
            category = 'dateTime'
        return self.request.locale.dates.getFormatter(
            category, 'short', None, u'gregorian')

    def getValue(self, obj):
        value = super(StartColumn, self).getValue(obj)
        if value and not obj.whole_day_event:
            value = value.astimezone(
                icemac.addressbook.preferences.utils.get_time_zone())
        return value

    def renderCell(self, obj):
        value = self.getValue(obj)
        if value:
            formatter = self.getFormatter(obj)
            value = formatter.format(value)
        return value


class Table(icemac.addressbook.browser.table.Table):
    """List recurring events."""

    title = icemac.addressbook.browser.breadcrumb.DO_NOT_SHOW
    no_rows_message = _(u'No recurring events defined yet.')

    def setUpColumns(self):
        return [
            z3c.table.column.addColumn(
                self, icemac.addressbook.browser.table.TitleLinkColumn,
                'title'),
            z3c.table.column.addColumn(
                self, StartColumn, 'datetime', weight=10),
            z3c.table.column.addColumn(
                self, RecurrenceColumn, 'period', weight=20),
            z3c.table.column.addColumn(
                self, icemac.addressbook.browser.table.DateColumn, 'end',
                header=_('to'), entity=icemac.addressbook.interfaces.IEntity(
                    icemac.ab.calendar.interfaces.IRecurringEvent),
                field=icemac.ab.calendar.interfaces.IRecurringEvent['end'],
                weight=30),
            z3c.table.column.addColumn(
                self, PersonsColumn, 'persons', weight=40),
            z3c.table.column.addColumn(
                self, icemac.addressbook.browser.table.TruncatedContentColumn,
                'notes', header=_('notes'), attrName='text', weight=50),
        ]

    @property
    def values(self):
        return self.context.values()


class Add(EventFields, icemac.addressbook.browser.base.BaseAddForm):
    """Add form for a recurring event."""

    title = _(u'Add new recurring event')
    interface = icemac.ab.calendar.interfaces.IRecurringEvent
    class_ = icemac.ab.calendar.event.RecurringEvent
    next_url = 'parent'


class Edit(EventFields, icemac.addressbook.browser.base.GroupEditForm):
    """Edit form for recurring event."""

    title = _(u'Edit recurring event')
    groups = (icemac.addressbook.browser.metadata.MetadataGroup,)
    interface = icemac.ab.calendar.interfaces.IRecurringEvent
    next_url = 'parent'
    z3c.form.form.extends(icemac.addressbook.browser.base.GroupEditForm,
                          ignoreFields=True)

    def applyChanges(self, data):
        # GroupForm has its own applyChanges but we need the one from
        # _AbstractEditForm here as inside the groups no changes are made but
        # there is a subscriber which raises an error which is handled by
        # _AbstractEditForm.
        return icemac.addressbook.browser.base._AbstractEditForm.applyChanges(
            self, data)

    @z3c.form.button.buttonAndHandler(
        _(u'Delete'), name='delete',
        condition=icemac.addressbook.browser.base.can_access('@@delete.html'))
    def handleDelete(self, action):
        self.redirect_to_next_url('object', 'delete.html')


class Delete(icemac.addressbook.browser.base.BaseDeleteForm):
    """Confirm delete of recurring event."""

    title = _('Delete recurring event')
    label = _('Do you really want to delete this recurring event?')
    interface = icemac.ab.calendar.interfaces.IRecurringEvent
    field_names = EVENT_CONFIRMATION_FIELDS



from __future__ import unicode_literals
from zope.security.interfaces import Unauthorized
import pytest


@pytest.fixture(scope='function')
def master_data_menu(address_book, browser, sitemenu):
    """Fixture to test the calendar master data menu."""
    browser.login('mgr')
    return sitemenu(browser, 4, 'Master data', browser.CALENDAR_MASTERDATA_URL)


def test_menu__master_data_menu__1(master_data_menu):
    """Asserting that the menu with the index 4 is calendar master data."""
    assert (master_data_menu.menu_item_title
            == master_data_menu.get_menu_item_title_under_test())


def test_menu__master_data_menu__2(master_data_menu):
    """The master data tab is selected on calendar master data overview."""
    assert master_data_menu.item_selected(master_data_menu.menu_item_URL)


def test_menu__master_data_menu__3(master_data_menu):
    """The master data tab is selected on event categories."""
    assert master_data_menu.item_selected(
        master_data_menu.browser.CALENDAR_CATEGORIES_LIST_URL)


def test_menu__master_data_menu__4(master_data_menu):
    """The master data tab is selected on edit display."""
    master_data_menu.item_selected(
        master_data_menu.browser.CALENDAR_MASTERDATA_EDIT_DISPLAY_URL)


def test_menu__CalendarMasterDataManager__1(address_book, browser):
    """It allows to navigate to the calendar master data."""
    browser.login('cal-visitor')
    browser.open(browser.MASTER_DATA_URL)
    browser.getLink('Calendar', index=1).click()
    assert browser.CALENDAR_MASTERDATA_URL == browser.url
    assert 'Event categories' in browser.contents


def test_menu__CalendarMasterDataManager__2(address_book, browser):
    """It prevents access for anonymous."""
    browser.handleErrors = False  # needed to catch exception
    with pytest.raises(Unauthorized):
        browser.open(browser.CALENDAR_MASTERDATA_URL)



from icemac.addressbook.principals.roles import has_editor_role
from icemac.addressbook.principals.roles import has_visitor_role


def test_roles__1(zcmlS):
    """The calendar editor role is registered as an editor role."""
    assert has_editor_role(['icemac.ab.calendar.Editor'])


def test_roles__2(zcmlS):
    """The calendar visitor role is registered as an visitor role."""
    assert has_visitor_role(['icemac.ab.calendar.Visitor'])



import zope.catalog.interfaces
from icemac.addressbook.i18n import _
from icemac.recurrence.interfaces import IRecurringDateTime
import collections
import datetime
import gocept.reference.field
import icemac.addressbook.fieldsource
import icemac.addressbook.interfaces
import icemac.addressbook.sources
import zc.sourcefactory.basic
import zope.cachedescriptors.property
import zope.component
import zope.interface


PACKAGE_ID = 'icemac.ab.calendar'
DATE_INDEX = 'icemac.ab.calendar.event.date'


class ICalendarObject(zope.interface.Interface):
    """Marker interface for objects belonging to the calendar.

    This is needed to get the calendar skin layer.

    """


class ICalendar(ICalendarObject,
                zope.location.interfaces.ILocation):
    """Calender and storage for dates."""

    def get_events_for_month(month, timezone=None):
        """Get all events which belong to `month` sorted by datetime.

        All events = single events and recurred events in month but no deleted
        events.

        month ... ``gocept.month.Month`` object.
        timezone ... str, None defaults to UTC.

        """

    def get_events(start, end, timezone=None):
        """Get all events which between `start` and `end` sorted by datetime.

        `start` belongs to the interval but `end` does not.
        All events = single events and recurred events in month but no deleted
        events.

        start ... ``datetime.datetime`` object.
        end ... ``datetime.datetime`` object.
        timezone ... str, None defaults to UTC.


        """


class EventFieldsSource(zc.sourcefactory.basic.BasicSourceFactory):
    """Fields of an event for display in calendar view."""

    @property
    def event_entity(self):
        return icemac.addressbook.interfaces.IEntity(IEvent)

    def getValues(self):
        # We need the default and user defined fields but not the ones which
        # are always displayed or handeled in a specfic way:
        for field_name, field in self.event_entity.getRawFields():
            if field_name in ('datetime', 'category', 'alternative_title',
                              'external_persons', 'whole_day_event'):
                continue
            yield field

    def getToken(self, value):
        return icemac.addressbook.fieldsource.tokenize(
            self.event_entity, value.__name__)

    def getTitle(self, value):
        # If we get here it is no security risk to return the title because the
        # user is allowed to access the value. But we did not register class
        # types and security assertions for fields.
        return zope.security.proxy.removeSecurityProxy(value).title


event_fields_source = EventFieldsSource()


class ICalendarDisplaySettings(zope.interface.Interface):
    """Configuration of how to display the calendar."""

    event_additional_fields = zope.schema.List(
        title=_('Additional event fields to be displayed in calendar'),
        required=False,
        value_type=zope.schema.Choice(source=event_fields_source))

    person_keyword = zope.schema.Choice(
        title=_('Person keyword'),
        description=_(
            'Only persons having the keyword assigned which is selected here '
            'will show up in the person list when editing an event. '
            'If no keyword is selected all persons will show up.'),
        source=icemac.addressbook.interfaces.keyword_source,
        required=False)


class ICalendarProvider(zope.interface.Interface):
    """Marker interface for objects providing a calendar on an attribute.

    This is necessary to meet security which otherwise raises a ForbiddenError.

    """

    calendar = zope.interface.Attribute(u'ICalendar')


class ICalendarMasterData(zope.interface.Interface):
    """Marker interface for objects providing a calendar master data

    This is necessary to meet security which otherwise raises a ForbiddenError.

    """

    calendar_categories = zope.interface.Attribute(u'ICategories')
    calendar_recurring_events = zope.interface.Attribute(u'IRecurringEvents')
    calendar_eventviews = zope.interface.Attribute(u'IEventViewContainer')


class ICategories(zope.interface.Interface):
    """Container for event categories."""


class ICategory(zope.interface.Interface):
    """An event category."""

    title = zope.schema.TextLine(title=_(u'event category'))


class CategorySource(zc.sourcefactory.basic.BasicSourceFactory):
    """Source of event categories defined for the calendar."""

    def getValues(self):
        categories = zope.component.getUtility(ICategories)
        return sorted(categories.values(), key=lambda x: x.title.lower())

    def getTitle(self, value):
        return value.title


category_source = CategorySource()


class PersonSource(zc.sourcefactory.basic.BasicSourceFactory):
    """Persons in addressbook."""

    def getValues(self):
        site = zope.component.hooks.getSite()
        kw = ICalendarDisplaySettings(ICalendar(site)).person_keyword
        if kw is None:
            values = site.values()
        else:
            catalog = zope.component.getUtility(
                zope.catalog.interfaces.ICatalog)
            values = catalog.searchResults(keywords={'all_of': [kw.title]})
        return values

    def getTitle(self, value):
        return icemac.addressbook.interfaces.ITitle(value)


person_source = PersonSource()


class IBaseEvent(zope.interface.Interface):
    """Base of single, recurring and recurred events."""

    category = zope.schema.Choice(
        title=_('event category'), source=category_source)
    datetime = zope.schema.Datetime(title=_('datetime'), required=True)
    whole_day_event = zope.schema.Bool(
        title=_('whole day event?'), default=False)
    whole_day_event.setTaggedValue('omit-from-field-list', True)
    alternative_title = zope.schema.TextLine(
        title=_('alternative title to category'), required=False)
    persons = gocept.reference.field.Set(
        title=_('persons'), required=False,
        value_type=zope.schema.Choice(
            title=_('persons'), source=person_source))
    # Cannot use Set of TextLine here, as it is not supported by z3c.form:
    external_persons = zope.schema.List(
        title=_('other persons'), required=False,
        value_type=zope.schema.TextLine(title=_('person name')))
    text = zope.schema.Text(title=_('notes'), required=False)

    def listPersons():
        """Sorted list of all persons incl. external ones as strings."""

    def in_timezone(timezone):
        """Date of event normalized to `timezone`.

        Whole day events are counted as midnight.

        timezone ... str

        """


class IEvent(IBaseEvent):
    """A single event in the calendar."""

    deleted = zope.interface.Attribute(
        'Event has been deleted. Used to support deletion of recurred events.')


class IEventDateTime(zope.interface.Interface):
    """Datetime of an event, used for cataloging the event."""

    datetime = zope.interface.Attribute('datetime of the event')


class IRecurringEvents(ICalendarObject):
    """Container for recurring events."""

    def get_events(categories):
        """Return the events of categories sorted by priority (ascending).

        categories ... list of category titles or `[]` for all events
        """


class RecurrencePeriodSource(icemac.addressbook.sources.TitleMappingSource):
    """Periods after which an event is repeated."""

    @zope.cachedescriptors.property.Lazy
    def _mapping(self):
        names_and_adapters = zope.component.getAdapters(
            [datetime.datetime.now()], IRecurringDateTime)
        return collections.OrderedDict(
            (name, adapter.title)
            for name, adapter in sorted(
                names_and_adapters, key=lambda x: x[1].weight))


recurrence_period_source = RecurrencePeriodSource()


class IRecurrence(zope.interface.Interface):
    """Time interval after which an event recurres."""

    period = zope.schema.Choice(
        title=_('recurrence period'), source=recurrence_period_source)


class IRecurringEventAdditionalSchema(IRecurrence):
    """Additional schema fields for IRecurringEvent."""

    end = zope.schema.Date(title=_('recurrence end'), required=False)


class IRecurringEvent(IBaseEvent, IRecurringEventAdditionalSchema):
    """An event recurring after a defined period."""

    priority = zope.interface.Attribute(
        'Weight of the selected recurrence period.')

    def get_events(interval_start, interval_end, timezone):
        """Get the events computed from recurrence in the datetime interval."""


class IRecurredEvent(IBaseEvent):
    """An event computed from IRecurringEvent."""

    __parent__ = zope.interface.Attribute('Calendar the event belongs to')
    recurring_event = zope.interface.Attribute(
        'RecurringEvent which was the source for the RecurredEvent.')

    def create_from(recurring_event, datetime):
        """Create an instance with data from the recurring event but for ...

        ... `datetime`.
        """


class INoSecurityProxyType(zope.interface.interfaces.IInterface):
    """"Marker interface for interfaces which require security unwrapping.

    The `.masterdata.calendar.AnnotationField` security unrwarps contexts
    providing such an interface because they are stored in annotations.

    Usage: register the interface in ZCML via

    <interface interface=".your.Interface"
               type="icemac.ab.calendar.interfaces.INoSecurityProxyType" />
    """



# -*- coding: utf-8 -*-
from icemac.ab.calendar.category import Category, CategoryContainer
from icemac.ab.calendar.interfaces import ICategory, ICategories
from zope.interface.verify import verifyObject
import zope.component.hooks


def test_category__Category__1():
    """It implements the `ICategory` interface."""
    assert verifyObject(ICategory, Category())


def test_category__Category____repr____1():
    """It returns a readable representation."""
    category = Category()
    category.title = u'Füchschen'
    # repr() has to be ASCII, so everything else is replaced:
    assert "<Category title='F?chschen'>" == repr(category)


def test_category__CategoryContainer__1():
    """It implements the `ICategories` interface."""
    assert verifyObject(ICategories, CategoryContainer())


def test_category__changed__1(
        address_book, CategoryFactory, EventFactory, DateTime, browser):
    """It updates the `keywords` index if a category title was changed."""
    cat = CategoryFactory(address_book, u'foo')
    EventFactory(address_book, category=cat, alternative_title=u'event',
                 datetime=DateTime.now)

    event_titles = [
        x.alternative_title
        for x in address_book.calendar.get_events(
            DateTime.add(DateTime.now, days=-1),
            DateTime.add(DateTime.now, days=+1),
            categories=[u'foo'],
        )]
    assert ['event'] == event_titles

    browser.login('mgr')
    browser.open(browser.CALENDAR_CATEGORY_EDIT_URL)
    browser.getControl('event category').value = 'bar'
    browser.getControl('Apply').click()

    with zope.component.hooks.site(address_book):
        event_titles = [
            x.alternative_title
            for x in address_book.calendar.get_events(
                DateTime.add(DateTime.now, days=-1),
                DateTime.add(DateTime.now, days=+1),
                categories=['bar'],
            )]
    assert ['event'] == event_titles



from __future__ import unicode_literals
import pytest


@pytest.fixture(scope='function')
def calendar_menu(address_book, browser, sitemenu):
    """Fixture to test the calendar menu."""
    browser.login('cal-editor')
    return sitemenu(browser, 0, 'Calendar', browser.CALENDAR_OVERVIEW_URL)


def test_menu__calendar_menu__1(calendar_menu):
    """Asserting that the menu with the index 0 is `Calendar`."""
    assert (calendar_menu.menu_item_title
            == calendar_menu.get_menu_item_title_under_test())


def test_menu__calendar_menu__2(calendar_menu):
    """The calendar tab is selected on the calendar overview."""
    assert calendar_menu.item_selected(calendar_menu.menu_item_URL)


def test_menu__calendar_menu__3(calendar_menu):
    """The calendar tab is not selected on master data."""
    assert not calendar_menu.item_selected(
        calendar_menu.browser.MASTER_DATA_URL)


def test_menu__calendar_menu__4(calendar_menu):
    """The calendar tab is selected on the calendar year view."""
    assert calendar_menu.item_selected(
        calendar_menu.browser.CALENDAR_YEAR_OVERVIEW_URL)


def test_menu__calendar_menu__5(address_book, calendar_menu, EventFactory):
    """The calendar tab is selected on the event clone view."""
    EventFactory(address_book)
    calendar_menu.item_selected(calendar_menu.browser.EVENT_CLONE_URL)


def test_menu__CalendarMenuItem__1(address_book, browser):
    """It allows to navigate the to calendar.

    The calendar view defaults to the month overview.
    """
    browser.login('cal-editor')
    browser.open(browser.ADDRESS_BOOK_DEFAULT_URL)
    assert browser.CALENDAR_OVERVIEW_URL == browser.getLink('Calendar').url
    browser.getLink('Calendar').click()
    assert browser.CALENDAR_MONTH_OVERVIEW_URL == browser.url



import icemac.addressbook.generations.utils
import pytz


@icemac.addressbook.generations.utils.evolve_addressbooks
def evolve(addressbook):
    """Fix tzinfo for DST and convert it to UTC."""
    for event in addressbook.calendar.values():
        if event.datetime is None:
            continue
        if event.datetime.tzinfo == pytz.utc:
            continue
        # fix UTC offset in DST + convert to UTC
        event.datetime = pytz.utc.normalize(
            event.datetime.tzinfo.localize(
                event.datetime.replace(tzinfo=None)))



# -*- coding: utf-8 -*-
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.support.wait import WebDriverWait
import datetime
import icemac.addressbook.testing
import icemac.recurrence.conftest
import pytz


class Browser(icemac.addressbook.testing.Browser):
    """Browser adapted for calendar."""

    BASE = 'http://localhost/ab/++attribute++calendar'
    CALENDAR_OVERVIEW_URL = BASE
    CALENDAR_MONTH_OVERVIEW_URL = BASE + '/@@month.html'
    CALENDAR_YEAR_OVERVIEW_URL = BASE + '/@@year.html'
    CALENDAR_EVENT_VIEWS_URL = BASE + '/@@event-view.html'

    CALENDAR_MASTERDATA_URL = 'http://localhost/ab/@@calendar-masterdata.html'
    CALENDAR_MASTERDATA_EDIT_DISPLAY_URL = BASE + '/@@edit-display.html'

    EV_BASE = BASE + '_eventviews'
    CALENDAR_MASTERDATA_EVENTVIEW_URL = EV_BASE
    CALENDAR_EVENTVIEW_CONFIGURATION_ADD_URL = (
        EV_BASE + '/@@addEventViewConfiguration.html')
    CALENDAR_EVENTVIEW_CONFIGURATION_EDIT_URL = (
        EV_BASE + '/EventViewConfiguration')
    CALENDAR_EVENTVIEW_CONFIGURATION_DELETE_URL = (
        EV_BASE + '/EventViewConfiguration/@@delete.html')

    CALENDAR_EVENT_FIELDS_LIST_URL = (
        'http://localhost/ab/++attribute++entities/'
        'icemac.ab.calendar.event.Event')

    CALENDAR_CATEGORIES_LIST_URL = BASE + '_categories'
    CALENDAR_CATEGORY_ADD_URL = BASE + '_categories/@@addEventCategory.html'
    CALENDAR_CATEGORY_EDIT_URL = BASE + '_categories/Category'
    CALENDAR_CATEGORY_DELETE_URL = BASE + '_categories/Category/@@delete.html'

    RE_BASE = BASE + '_recurring_events'
    CALENDAR_RECURRING_EVENTS_LIST_URL = RE_BASE
    CALENDAR_RECURRING_EVENT_ADD_URL = RE_BASE + '/@@addRecurringEvent.html'
    CALENDAR_RECURRING_EVENT_EDIT_URL = RE_BASE + '/RecurringEvent'
    CALENDAR_RECURRING_EVENT_DELETE_URL = (
        RE_BASE + '/RecurringEvent/@@delete.html')

    EVENT_ADD_URL = BASE + '/@@addEvent.html'
    EVENT_EDIT_URL = BASE + '/Event'
    EVENT_CLONE_URL = BASE + '/Event/@@clone.html'
    EVENT_DELETE_URL = BASE + '/Event/@@delete.html'

    RECURRED_EVENT_CUSTOMIZE_URL = BASE + '/@@customize-recurred-event'
    RECURRED_EVENT_ADD_URL = BASE + '/@@addFromRecurredEvent.html'
    RECURRED_EVENT_VIEW_URL = BASE + '/@@viewRecurredEvent.html'
    RECURRED_EVENT_DELETE_URL = BASE + '/@@delete-recurred-event.html'

    @property
    def ucontents(self):
        """Browser contents decoded to unicode."""
        return self.contents.decode('utf-8')


class CalendarWebdriverPageObjectBase(
        icemac.addressbook.testing.WebdriverPageObjectBase):
    """Base for page object classes to used with to ``Webdriver.attach()``."""

    browser = Browser


class POCalendar(CalendarWebdriverPageObjectBase):
    """Webdriver page object for the calendar itself."""

    paths = [
        'CALENDAR_MONTH_OVERVIEW_URL',
        'CALENDAR_YEAR_OVERVIEW_URL',
    ]

    @property
    def month(self):
        return self.get_drop_down_selection('form-widgets-calendar_month-row')

    @month.setter
    def month(self, month):
        self.select_from_drop_down(month, id='form-widgets-calendar_month-row')
        self._wait_for_info_message()

    def switch_to_previous_month(self):
        self._selenium.find_element_by_link_text(u"◄").click()

    def switch_to_next_month(self):
        self._selenium.find_element_by_link_text(u"►").click()

    @property
    def year(self):
        return int(
            self.get_drop_down_selection('form-widgets-calendar_year-row'))

    @year.setter
    def year(self, year):
        self.select_from_drop_down(year, id='form-widgets-calendar_year-row')
        self._wait_for_info_message()

    def _wait_for_info_message(self):
        WebDriverWait(self._selenium, 10).until(
            expected_conditions.invisibility_of_element_located(
                (By.ID, 'info-messages')))


icemac.addressbook.testing.Webdriver.attach(POCalendar, 'calendar')


def get_recurred_event(recurring_event, DateTime):
    """Get one recurred event at today.

    DateTime ... the DateTime fixture instance.
    """
    return recurring_event.get_events(
        DateTime.today_8_32_am,
        DateTime.add(DateTime.today_8_32_am, days=1),
        pytz.utc).next()


# Fixture helpers


class DateTimeClass(icemac.recurrence.conftest.DateTimeClass,
                    icemac.addressbook.testing.DateTimeClass):
    """Helper class to create and format datetime objects."""

    @property
    def today_8_32_am(self):
        """Get a datetime object for today with fixed time."""
        return datetime.datetime.combine(
            datetime.date.today(), datetime.time(8, 32, tzinfo=pytz.utc))



# -*- coding: utf-8 -*-
import setuptools


def read(path):
    """Read file."""
    with open(path) as f:
        return f.read()


version = '3.1'
long_description = '\n\n'.join([
    read('README.rst'),
    read('CHANGES.rst'),
])

setuptools.setup(
    name='icemac.ab.calendar',
    version=version,
    description="Calendar feature for icemac.addressbook",
    long_description=long_description,
    keywords='icemac addressbook calendar event recurring',
    author='Michael Howitz',
    author_email='icemac@gmx.net',
    download_url='https://pypi.org/project/icemac.ab.calendar',
    url='https://bitbucket.org/icemac/icemac.ab.calendar',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Zope3',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Natural Language :: German',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 2 :: Only',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    packages=setuptools.find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['icemac', 'icemac.ab'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'Pyphen',
        'decorator',
        'gocept.month >= 1.2',
        'grokcore.annotation',
        'grokcore.component >= 2.5.1.dev1',
        'icemac.ab.locales >= 2.16',
        'icemac.addressbook >= 7.0.dev0',
        'icemac.recurrence >= 1.3.1.dev0',
        'js.classy',
        'js.bootstrap4',
        'setuptools',
        'z3c.form >= 3.3',
        'zope.cachedescriptors',
        'zope.securitypolicy >= 4.1',
    ],
    extras_require=dict(
        test=[
            'gocept.testing',
            'icemac.addressbook [test]',
            'zope.testing >= 3.8.0',
        ]),
    entry_points="""
      [fanstatic.libraries]
      calendar = icemac.ab.calendar.browser.resource:lib
      """,
)



from icemac.ab.calendar.interfaces import ICalendarDisplaySettings, ICalendar
import icemac.addressbook.generations.utils


@icemac.addressbook.generations.utils.evolve_addressbooks
def evolve(addressbook):
    """Set default value of `person_keyword` in CalendarDisplaySettings."""
    ICalendarDisplaySettings(ICalendar(addressbook)).person_keyword = None



from gocept.month import Month
from icemac.ab.calendar.calendar import Calendar, CalendarDisplaySettings
from icemac.ab.calendar.event import get_event_data_from_recurring_event
from icemac.ab.calendar.interfaces import ICalendar, ICalendarDisplaySettings
from zope.interface.verify import verifyObject
import pytest
import pytz


def test_calendar__Calendar__1():
    """It fulfills the `ICalendar` interface."""
    assert verifyObject(ICalendar, Calendar())


def test_calendar__CalendarDisplaySettings__1():
    """It fulfills the `ICalendarDisplaySettings` interface."""
    assert verifyObject(ICalendarDisplaySettings, CalendarDisplaySettings())


@pytest.fixture('function')
def sample_events(address_book, CategoryFactory, EventFactory, DateTime):
    """Fixture providing some example events for tests."""
    foo = CategoryFactory(address_book, u'foo')
    bar = CategoryFactory(address_book, u'bar')
    # The order of creation is important to test that the results are sorted:
    EventFactory(address_book, alternative_title=u'start Mar 2013',
                 category=foo,
                 datetime=DateTime(2013, 3, 1, 0))
    EventFactory(address_book, alternative_title=u'start Feb 2013',
                 category=bar,
                 datetime=DateTime(2013, 2, 1, 0))
    EventFactory(address_book, alternative_title=u'end Jan 2013',
                 datetime=DateTime(2013, 1, 31, 23, 59))
    EventFactory(address_book, alternative_title=u'end Feb 2013',
                 category=foo,
                 datetime=DateTime(2013, 2, 28, 23, 59))
    return address_book


def test_calendar__Calendar__get_events_for_month__1(sample_events):
    """It returns only events in the given month."""
    assert ([u'start Feb 2013', u'end Feb 2013'] ==
            [x.alternative_title
             for x in sample_events.calendar.get_events_for_month(
                 Month(2, 2013))])


def test_calendar__Calendar__get_events_for_month__2(sample_events):
    """It respects the given time zone for an eastern time zone."""
    assert ([u'end Feb 2013', u'start Mar 2013'] ==
            [x.alternative_title
             for x in sample_events.calendar.get_events_for_month(
                 Month(2, 2013), 'Etc/GMT+1')])


def test_calendar__Calendar__get_events_for_month__3(sample_events):
    """It respects the given time zone for a western time zone."""
    assert ([u'end Jan 2013', u'start Feb 2013'] ==
            [x.alternative_title
             for x in sample_events.calendar.get_events_for_month(
                 Month(2, 2013), 'Etc/GMT-1')])


def test_calendar__Calendar__get_events_for_month__4(
        sample_events, address_book, RecurringEventFactory, CategoryFactory,
        DateTime):
    """It returns the recurring events in the given month."""
    RecurringEventFactory(
        address_book,
        alternative_title=u'each week',
        datetime=DateTime(2013, 2, 14, 23, 0),
        period=u'weekly',
        category=CategoryFactory(address_book, u'night lunch'))
    RecurringEventFactory(
        address_book,
        alternative_title=u'each week in future',
        datetime=DateTime(2013, 3, 1, 0),
        period=u'weekly',
        category=CategoryFactory(address_book, u'midnight'))
    assert ([(u'end Jan 2013', '2013-01-31T23:59:00+00:00'),
             (u'start Feb 2013', '2013-02-01T00:00:00+00:00'),
             (u'each week', '2013-02-15T00:00:00+01:00'),
             (u'each week', '2013-02-22T00:00:00+01:00')] ==
            [(x.alternative_title, x.datetime.isoformat())
             for x in address_book.calendar.get_events_for_month(
                 Month(2, 2013), 'Etc/GMT-1')])


def test_calendar__Calendar__get_events_for_month__5(
        address_book, CategoryFactory, RecurringEventFactory, EventFactory,
        DateTime):
    """A customized recurred event hides the original recurred event."""
    recurring_event = RecurringEventFactory(
        address_book,
        alternative_title=u'each week',
        datetime=DateTime(2013, 3, 14, 23, 0),
        period=u'weekly',
        category=CategoryFactory(address_book, u'night lunch'))
    event_data = get_event_data_from_recurring_event(
        recurring_event, DateTime(2013, 3, 21, 23, 0))
    event_data['alternative_title'] = u'this week'
    EventFactory(address_book, **event_data)
    assert ([(u'each week', '2013-03-14'),
             (u'this week', '2013-03-21'),
             (u'each week', '2013-03-28')] ==
            [(x.alternative_title, x.datetime.date().isoformat())
             for x in address_book.calendar.get_events_for_month(
                 Month(3, 2013), 'Etc/GMT+1')])


def test_calendar__Calendar__get_events_for_month__6(
        address_book, CategoryFactory, RecurringEventFactory, DateTime):
    """The recurring event with the higher prio in the category is rendered."""
    category = CategoryFactory(address_book, u'night lunch')
    RecurringEventFactory(
        address_book,
        alternative_title=u'monthly lunch',
        datetime=DateTime(2013, 3, 10, 12),
        period=u'nth weekday of month',
        category=category)
    RecurringEventFactory(
        address_book,
        alternative_title=u'weekly lunch',
        datetime=DateTime(2013, 3, 3, 12),
        period=u'weekly',
        category=category)
    assert ([(u'weekly lunch', '2013-03-03'),
             (u'monthly lunch', '2013-03-10'),
             (u'weekly lunch', '2013-03-17'),
             (u'weekly lunch', '2013-03-24'),
             (u'weekly lunch', '2013-03-31')] ==
            [(x.alternative_title, x.datetime.date().isoformat())
             for x in address_book.calendar.get_events_for_month(
                 Month(3, 2013), 'Etc/GMT+1')])


def test_calendar__Calendar__get_events_for_month__7(
        sample_events, address_book, EventFactory, DateTime):
    """It does not return deleted events."""
    event = EventFactory(
        address_book,
        alternative_title=u'deleted start of March 2013',
        datetime=DateTime(2013, 3, 1, 5))
    event.deleted = True
    assert ([u'start Mar 2013'] ==
            [x.alternative_title
             for x in address_book.calendar.get_events_for_month(
                 Month(3, 2013), 'Etc/GMT')])


def test_calendar__Calendar__get_events_for_month__8(
        address_book, EventFactory, DateTime):
    """A whole day event does not change the month via timezone."""
    EventFactory(address_book,
                 alternative_title=u'whole day',
                 datetime=DateTime(2015, 4, 30, 23),
                 whole_day_event=True)
    EventFactory(address_book,
                 alternative_title=u'day',
                 datetime=DateTime(2015, 4, 30, 23))
    assert ([u'whole day', u'day'] ==
            [x.alternative_title
             for x in address_book.calendar.get_events_for_month(
                 Month(4, 2015), 'Etc/GMT+8')])
    assert ([u'whole day'] ==
            [x.alternative_title
             for x in address_book.calendar.get_events_for_month(
                 Month(4, 2015), 'Etc/GMT-7')])


def test_calendar__Calendar__get_events_for_month__9(
        address_book, EventFactory, CategoryFactory, DateTime):
    """Deleted events are depressed in favour of not deleted ones.

    This behaviour requires the events to have the same category and the same
    time. It is needed to be able to delete a recurring event and add a new one
    later at the same place for the same category.
    """
    category = CategoryFactory(address_book, u'my cat')
    date = DateTime(2017, 5, 14, 13, 50, tzinfo=pytz.timezone('Europe/Berlin'))
    e1 = EventFactory(address_book, category=category, datetime=date)
    e1.deleted = True
    EventFactory(address_book, category=category, datetime=date)
    assert ([False] == [x.deleted
                        for x in address_book.calendar.get_events_for_month(
                            Month(5, 2017), 'Europe/Berlin')])


def test_calendar__Calendar__get_events__1(sample_events, DateTime):
    """It returns only events in the given time range."""
    assert ([u'end Jan 2013', u'start Feb 2013', u'end Feb 2013'] ==
            [x.alternative_title
             for x in sample_events.calendar.get_events(
                 DateTime(2013, 1, 30, 0), DateTime(2013, 3, 1, 0))])


def test_calendar__Calendar__get_events__2(sample_events, DateTime):
    """It returns only events for the given categories. (single category)"""
    assert ([u'end Feb 2013', u'start Mar 2013'] ==
            [x.alternative_title
             for x in sample_events.calendar.get_events(
                 DateTime(2013, 1, 1, 0), DateTime(2013, 3, 31, 0),
                 categories=['foo'])
             ])


def test_calendar__Calendar__get_events__3(
        sample_events, CategoryFactory, RecurringEventFactory, DateTime):
    """It returns only events for the given categories. (many categories)"""
    RecurringEventFactory(
        sample_events,
        category=CategoryFactory(sample_events, u'month'),
        alternative_title=u'each month',
        datetime=DateTime(2013, 2, 14, 23, 0),
        period=u'nth weekday of month')

    RecurringEventFactory(
        sample_events,
        category=CategoryFactory(sample_events, u'week'),
        alternative_title=u'each week',
        datetime=DateTime(2013, 1, 14, 23, 0),
        period=u'weekly')

    requested_categories = ['foo', 'bar', 'month']
    assert ([u'start Feb 2013',
             u'each month',
             u'end Feb 2013',
             u'start Mar 2013',
             u'each month',
             ] == [x.alternative_title
                   for x in sample_events.calendar.get_events(
                       DateTime(2013, 1, 1, 0), DateTime(2013, 3, 31, 0),
                       categories=requested_categories)])



from .interfaces import ICalendarMasterData
import icemac.ab.calendar.interfaces
import icemac.addressbook.browser.menus.menu
import z3c.menu.ready2go.manager
import zope.viewlet.manager


# Menu displaying the links to the edit the calendar master data
CalendarMasterDataManager = zope.viewlet.manager.ViewletManager(
    'calendar-master-data', ICalendarMasterData, bases=(
        z3c.menu.ready2go.manager.MenuManager,))


calendar_views = icemac.addressbook.browser.menus.menu.SelectMenuItemOn(
    'calendar-masterdata.html',
    icemac.ab.calendar.interfaces.ICategories,
    icemac.ab.calendar.interfaces.ICategory,
    icemac.ab.calendar.interfaces.IRecurringEvents,
    icemac.ab.calendar.interfaces.IRecurringEvent,
    'edit-display.html')



import icemac.addressbook.generations.utils


@icemac.addressbook.generations.utils.evolve_addressbooks
def evolve(addressbook):
    """Set permissions on users again to allow them to edit themselves."""
    for principal in addressbook.principals.values():
        # Side effect: set entries in permission map to allow user to edit his
        # own data.
        principal.roles = principal.roles



import grokcore.component as grok
import icemac.addressbook.browser.breadcrumb
import zope.component


class CalendarMDChildBreadcrumb(
        icemac.addressbook.browser.breadcrumb.Breadcrumb):
    """Base class for a breadcrumb those parent is calendar master data."""

    grok.baseclass()

    @property
    def parent(self):
        return zope.component.getMultiAdapter(
            (icemac.addressbook.interfaces.IAddressBook(self.context),
             self.request),
            name='calendar-masterdata.html')



from datetime import datetime, time
from icemac.addressbook.i18n import _
from icemac.addressbook.interfaces import IField
import gocept.reference
import grokcore.component as grok
import icemac.ab.calendar.interfaces
import icemac.addressbook.entities
import icemac.addressbook.interfaces
import icemac.addressbook.utils
import icemac.recurrence
import itertools
import persistent
import pytz
import zope.annotation.interfaces
import zope.container.btree
import zope.container.contained
import zope.interface


MIDNIGHT = time(0, 0)
NOON = time(12, 0)


class BaseEvent(object):
    """Base class of all event classes."""

    def listPersons(self):
        """Sorted list of all persons incl. external ones as strings."""
        return sorted(itertools.chain(
            [icemac.addressbook.interfaces.IPersonName(x).get_name()
             for x in (self.persons or [])],
            (self.external_persons or [])))

    def in_timezone(self, timezone):
        """Date of event normalized to `timezone`.

        Whole day events are counted as midnight.

        """
        if self.whole_day_event:
            return timezone.localize(
                datetime.combine(self.datetime.date(), MIDNIGHT))
        return timezone.normalize(self.datetime)


event_schema = icemac.ab.calendar.interfaces.IEvent


@zope.interface.implementer(
    event_schema,
    icemac.addressbook.interfaces.ISchemaProvider,
    zope.annotation.interfaces.IAttributeAnnotatable)
class Event(persistent.Persistent,
            zope.container.contained.Contained,
            BaseEvent):
    """An event in the calendar."""

    schema = event_schema

    zope.schema.fieldproperty.createFieldProperties(
        icemac.ab.calendar.interfaces.IEvent, omit=['category', 'persons'])

    category = gocept.reference.Reference('category', ensure_integrity=True)
    persons = gocept.reference.ReferenceCollection(
        'persons', ensure_integrity=True)
    deleted = False

    def __init__(self):
        # prevent AttributeErrors on first read
        self.category = None
        self.persons = None

    def __repr__(self):
        """Custom repr to look inside the event for debugging purposes."""
        fstr = (
            "<{0.__class__.__name__}"
            " datetime='{0.datetime}'"
            " title={1!r},"
            " deleted={0.deleted}>"
        )
        return fstr.format(
            self, icemac.addressbook.interfaces.ITitle(self, '<unknown>'))


event_entity = icemac.addressbook.entities.create_entity(
    _(u'event'), icemac.ab.calendar.interfaces.IEvent, Event)


@grok.adapter(icemac.ab.calendar.interfaces.IEvent)
@grok.implementer(icemac.addressbook.interfaces.ITitle)
def event_title(event):
    """Human readable title for an event."""
    if event.alternative_title:
        return event.alternative_title
    if event.category:
        return icemac.addressbook.interfaces.ITitle(event.category)
    return _('event')


@grok.adapter(icemac.ab.calendar.interfaces.IEvent)
@grok.implementer(icemac.ab.calendar.interfaces.ICalendar)
def calendar_of_IEvent(event):
    """Adapt the event to its calendar."""
    return event.__parent__


@grok.adapter(icemac.ab.calendar.interfaces.IRecurredEvent)
@grok.implementer(icemac.ab.calendar.interfaces.ICalendar)
def calendar_of_IRecurredEvent(event):
    """Adapt the event to its calendar."""
    return event.__parent__


class EventDateTime(grok.Adapter):
    """Catalog datetime of event."""

    grok.context(icemac.ab.calendar.interfaces.IEvent)
    grok.implements(icemac.ab.calendar.interfaces.IEventDateTime)

    @property
    def datetime(self):
        """Date of the event always having a time for indexing."""
        if self.context.datetime is None:
            return None
        if self.context.whole_day_event:
            return pytz.utc.localize(
                datetime.combine(self.context.datetime.date(), NOON))
        return self.context.datetime


@zope.interface.implementer(icemac.ab.calendar.interfaces.IRecurringEvents)
class RecurringEventContainer(zope.container.btree.BTreeContainer):
    """A container for recurring events."""

    def get_events(self, categories):
        events = self.values()
        if categories:
            events = [x for x in events if x.category.title in categories]
        return sorted(events, key=lambda x: x.priority)


recurring_event_schema = icemac.ab.calendar.interfaces.IRecurringEvent


@zope.interface.implementer(recurring_event_schema)
class RecurringEvent(Event):
    """An event which repeats after a defined period."""

    schema = recurring_event_schema
    zope.schema.fieldproperty.createFieldProperties(
        icemac.ab.calendar.interfaces.IRecurringEventAdditionalSchema)

    def get_events(self, interval_start, interval_end, timezone):
        """Get the events computed from recurrence in the interval."""
        if self.end is not None:
            end_datetime = self._get_end_datetime(interval_end.tzinfo)
            if end_datetime < interval_end:
                interval_end = end_datetime
        recurrence_dates = icemac.recurrence.get_recurrences(
            timezone.normalize(self.datetime), self.period,
            interval_start, interval_end)
        for dt in recurrence_dates:
            yield RecurredEvent.create_from(self, dt)

    @property
    def priority(self):
        return icemac.recurrence.get_recurring(
            self.datetime, self.period).weight

    def _get_end_datetime(self, timzone):
        return datetime.combine(self.end, time(23, 59, 59, tzinfo=timzone))


recurring_event_entity = icemac.addressbook.entities.create_entity(
    _(u'recurring event'), icemac.ab.calendar.interfaces.IRecurringEvent,
    RecurringEvent)


def _get_field_name_on_IEvent(field, event_entity):
    """Get the name of the appropriate field on IEvent."""
    if not IField.providedBy(field):
        if field.__name__ in icemac.ab.calendar.interfaces.IBaseEvent:
            return field.__name__  # common field
        return None  # no-common field
    for name, recurring_field in event_entity.getRawFields(sorted=False):
        if (IField.providedBy(recurring_field) and
                field.title == recurring_field.title and
                field.type == recurring_field.type and
                field.values == recurring_field.values):
            return name
    return None


def get_event_data_from_recurring_event(recurring_event, datetime):
    """Get the event data from a recurring event

    This data can be written as the `__dict__` of the event.

    """
    event_entity = icemac.addressbook.interfaces.IEntity(
        icemac.ab.calendar.interfaces.IEvent)
    revent_entity = icemac.addressbook.interfaces.IEntity(
        icemac.ab.calendar.interfaces.IRecurringEvent)
    data = {}
    for name, field in revent_entity.getRawFields(sorted=False):
        key = _get_field_name_on_IEvent(field, event_entity)
        if key is not None:
            bound_field = icemac.addressbook.entities.get_bound_schema_field(
                recurring_event, revent_entity, field)
            data[key] = bound_field.get(bound_field.context)
    data[u'datetime'] = datetime
    return data


@grok.adapter(icemac.ab.calendar.interfaces.IRecurringEvent)
@grok.implementer(icemac.ab.calendar.interfaces.IEventDateTime)
def recurring_event_datetime(event):
    """Do not catalog datetime of recurring event."""
    return None


@grok.adapter(icemac.ab.calendar.interfaces.IRecurringEvent)
@grok.implementer(icemac.ab.calendar.interfaces.ICalendar)
def calendar_of_recurring_event(recurring_event):
    """Adapt the recurring event to its calendar."""
    return icemac.ab.calendar.interfaces.ICalendar(
        icemac.addressbook.interfaces.IAddressBook(None))


@zope.interface.implementer(
    icemac.ab.calendar.interfaces.IRecurredEvent,
    icemac.addressbook.interfaces.IUserFieldStorage)
class RecurredEvent(BaseEvent):
    """An event computed from RecurringEvent."""

    zope.schema.fieldproperty.createFieldProperties(
        icemac.ab.calendar.interfaces.IRecurredEvent)

    __parent__ = None
    recurring_event = None
    deleted = False

    @classmethod
    def create_from(cls, recurring_event, datetime):
        """Constructor: Copy data from recurring event."""
        data = get_event_data_from_recurring_event(recurring_event, datetime)
        data.update({
            '__parent__': icemac.ab.calendar.interfaces.ICalendar(
                recurring_event),
            'recurring_event': recurring_event})

        return icemac.addressbook.utils.create_obj(cls, **data)

    def __repr__(self):
        """Custom repr to look inside the recurred event to debug it."""
        return "<RecurredEvent datetime='{0.datetime}' title={1!r}>".format(
            self, icemac.addressbook.interfaces.ITitle(self, '<unknown>'))


@grok.adapter(icemac.ab.calendar.interfaces.IRecurredEvent)
@grok.implementer(icemac.addressbook.interfaces.ITitle)
def recurred_event_title(event):
    return event_title(event)


@grok.implementer(icemac.addressbook.interfaces.IKeywordTitles)
class EventKeywords(grok.Adapter):
    """Event categories to be stored in the `keywords` index in the catalog."""

    grok.context(icemac.ab.calendar.interfaces.IBaseEvent)

    def __init__(self, context):
        self.context = context

    def get_titles(self):
        return [icemac.addressbook.interfaces.ITitle(self.context.category)]
